#include "MainComponent.h"

MainComponent::MainComponent()
{
    setSize (800, 600);

    initializeAudio();
}

MainComponent::~MainComponent()
{
    shutdownAudio();
}

void MainComponent::initializeAudio()
{
    // Some platforms require permissions to open input channels so request that here
    if (juce::RuntimePermissions::isRequired (juce::RuntimePermissions::recordAudio)
        && ! juce::RuntimePermissions::isGranted (juce::RuntimePermissions::recordAudio))
    {
        juce::RuntimePermissions::request(juce::RuntimePermissions::recordAudio,
                                          [&] (bool granted) { setAudioChannels (granted ? 2 : 0, 2); });
    }
    else
    {
        // Specify the number of input and output channels that we want to open
        setAudioChannels(2, 2);
    }
    
    // If requested sample buffer is available, reconfigure the audio buffer size to use that instead
    constexpr int PREFERRED_DEFAULT_BUFFER_SIZE = 256;
    auto          buffer_size_list = deviceManager.getCurrentAudioDevice()->getAvailableBufferSizes();

    for (auto x : buffer_size_list)
    {
        juce::String text("Supported buffer size: ");
        text << x;
        DBG(text);
    }

    if (buffer_size_list.contains(PREFERRED_DEFAULT_BUFFER_SIZE))
    {
        juce::String text("SWITCHING TO BUFFER SIZE: ");
        text << PREFERRED_DEFAULT_BUFFER_SIZE;
        DBG(text);

        juce::AudioDeviceManager::AudioDeviceSetup current_audio_setup;

        deviceManager.getAudioDeviceSetup(current_audio_setup);
        current_audio_setup.bufferSize = PREFERRED_DEFAULT_BUFFER_SIZE;
        //current_audio_setup.sampleRate = 44100;

        deviceManager.setAudioDeviceSetup(current_audio_setup, true);
    }
}

void MainComponent::prepareToPlay (int samplesPerBlockExpected, double sampleRate)
{
}

static int s_previous_time = 0;
static int s_counter       = 0;

void MainComponent::getNextAudioBlock (const juce::AudioSourceChannelInfo& bufferToFill)
{
    const int current_time_block_start  = juce::Time::getMillisecondCounter();
    const int delta_time                = current_time_block_start - s_previous_time;
    
    if (delta_time == 0)
    {
        juce::String text("0ms block count: ");
        text << s_counter;
        DBG(text);

        s_counter++;
    }
    
    s_previous_time = current_time_block_start;


    bufferToFill.clearActiveBufferRegion();
}

void MainComponent::releaseResources()
{
}

void MainComponent::paint (juce::Graphics& g)
{
    g.fillAll (getLookAndFeel().findColour (juce::ResizableWindow::backgroundColourId));
}

void MainComponent::resized()
{
}
